[Reflection.Assembly]::LoadWithPartialName("System.Net.Http")
[Reflection.Assembly]::LoadWithPartialName("System.Web")

$merchantId = ""
$merchantHashSeed = ""
$dummyPaymentFlag = $false

# Normal SMTP
$smtpServerAddress = ""
$smtpEnableSsl = $false
$smtpServerPort = 25
$smtpNeedCredentials = $false
$smtpUser = ""
$smtpPassword = ''

# Auth SMTP
#$smtpServerAddress = ""
#$smtpEnableSsl = $true
#$smtpServerPort = 587
#$smtpNeedCredentials = $true
#$smtpUser = ""
#$smtpPassword = ''

$mailFrom = "noreply@veritrans.jp"
$mailSubject = "お支払いURLのご案内"
$mailBody = @'
%NAME% 様

サンプルショップをご利用いただき、ありがとうございます。

注文内容
----------------------
取引ID：%ORDERID%
金額：%AMOUNT%円
注文日：%ORDERDATE%
----------------------

下記のURLから、決済手続きを開始してください。
%PAYMENTLINK%
'@

$useProxy = $false
$useProxyCredential = $false

$apiHost = "https://pay3.veritrans.co.jp"
$apiPath = "/web1/commodityRegist.action"

$responseType = ""
$finishPaymentAccessUrl = ""


function SendMail($subject, $body, $to) {
    try {
        $Error.Clear()
        if([String]::IsNullOrEmpty($smtpServerAddress) -eq $false){
        $client = New-Object System.Net.Mail.SmtpClient($smtpServerAddress, $smtpServerPort)
        $client.EnableSsl = $smtpEnableSsl
        if ($smtpNeedCredentials -eq $true) {
            $client.Credentials = New-Object System.Net.NetworkCredential($smtpUser, $smtpPassword)
        }
        $message = New-Object System.Net.Mail.MailMessage($mailFrom, $to, $subject, $body)
        $client.Send($message)
        }
        return $true
    }
    catch [Exception] {
        return $false
    }
}

function CreateMailBody($body, $orderId, $paymentLink, $name, $amount, $orderDate, $expireDate, $expireTime) {
    $body = $body.Replace("%ORDERID%", $orderId)
    $body = $body.Replace("%PAYMENTLINK%", $paymentLink)
    $body = $body.Replace("%NAME%", $name)
    $body = $body.Replace("%AMOUNT%", $amount)
    $body = $body.Replace("%ORDERDATE%", $orderDate)
    $body = $body.Replace("%EXPIREDATE%", $expireDate)
    $body = $body.Replace("%EXPIRETIME%", $expireTime)
    return $body
}

function CreateMerchantHash($merchantHashSeed, $merchantId, $settlementType, $orderId, $amount) {
    $str = $merchantHashSeed + "," + $merchantId + "," + $settlementType + "," + $orderId + "," + $amount
    $sha512 = New-Object System.Security.Cryptography.SHA512Managed
    $utf8 = New-Object System.Text.UTF8Encoding
    $bytes = $utf8.GetBytes($str)
    $hex = $sha512.ComputeHash($bytes)
    return ($hex | % {$_.ToString("x2")}) -join ""
}

function GetPostBody([System.Collections.Generic.Dictionary[string, string]]$dic) {
    [string]$params = ""
    foreach ($key in $dic.Keys) {
        $params += $key + "=" + [System.Web.HttpUtility]::UrlEncode($dic[$key]) + "&"
    }
    $params = $params.Substring(0, $params.Length - 1)
    return $params
}

function GetParameterDictionary(
    [string]$merchantId, [string]$orderId, [string]$merchantHash, [string]$sessionId, [string]$amount, [String]$linkExpDatetime, [string]$settlementType,
    [string]$settlementSubType, [string]$cardCaptureFlag, [string]$shopName, [string]$screenTitle, [string]$contents, [string]$contentsKana,
    [string]$timelimitOfPayment, [string]$customerMailAddress, [string]$paymentMail) {
    $dic = New-Object 'System.Collections.Generic.Dictionary[string, string]'
    $dic.Add("MERCHANT_ID", $merchantId)
    $dic.Add("MERCHANTHASH", $merchantHash)
    if ([String]::IsNullOrEmpty($settlementType) -eq $false) {
        $dic.Add("SETTLEMENT_TYPE", $settlementType)
    }
    if ([String]::IsNullOrEmpty($settlementSubType) -eq $false) {
        $dic.Add("SETTLEMENT_SUB_TYPE", $settlementSubType)
    }
    $dic.Add("ORDER_ID", $orderId)
    $dic.Add("SESSION_ID", $sessionId)
    $dic.Add("AMOUNT", $amount)

    if ([String]::IsNullOrEmpty($responseType) -eq $false) {
        $dic.Add("RESPONSE_TYPE", $responseType)
    }
    if ([String]::IsNullOrEmpty($linkExpDatetime) -eq $false) {
        $dic.Add("LINK_EXP_DATETIME", $linkExpDatetime)
    }
    $dic.Add("LINK_PAYMENT_FLAG", "1")
    if ($paymentMail -eq "1") {
        $dic.Add("PAYMENT_MAIL_CONSUMER_ADDRESS", $customerMailAddress)
    }
    if ($dummyPaymentFlag -eq $true) {
        $dic.Add("DUMMY_PAYMENT_FLAG", "1")
    }
    if ([String]::IsNullOrEmpty($finishPaymentAccessUrl -eq $false)) {
        $dic.Add("FINISH_PAYMENT_ACCESS_URL", $finishPaymentAccessUrl)
    }
    if ([String]::IsNullOrEmpty($cardCaptureFlag) -eq $false) {
        $dic.Add("CARD_CAPTURE_FLAG", $cardCaptureFlag)
    }
    if ([String]::IsNullOrEmpty($shopName) -eq $false) {
        $dic.Add("SHOP_NAME", $shopName)
    }
    if ([String]::IsNullOrEmpty($screenTitle) -eq $false) {
        $dic.Add("SCREEN_TITLE", $screenTitle)
    }
    if ([String]::IsNullOrEmpty($contents) -eq $false) {
        $dic.Add("CONTENTS", $contents)
    }
    if ([String]::IsNullOrEmpty($contentsKana) -eq $false) {
        $dic.Add("CONTENTS_KANA", $contentsKana)
    }
    if ([String]::IsNullOrEmpty($timelimitOfPayment) -eq $false) {
        $dic.Add("TIMELIMIT_OF_PAYMENT", $timelimitOfPayment)
    }
    return $dic
}

function GetProxy([System.Uri]$target, [pscredential]$credentials) {
    if ([System.Net.WebRequest]::GetSystemWebProxy().IsBypassed($target) -eq $false) {
        $proxy = New-Object -TypeName System.Net.WebProxy @([System.Net.WebRequest]::GetSystemWebProxy().GetProxy($target))
        $proxy.Credentials = New-Object System.Net.NetworkCredential @($credentials.UserName, $credentials.Password)
        return $proxy
    }
}

function GetClient([System.Net.WebProxy]$proxy) {
    $httpClient = $NULL
    [System.Net.Http.HttpClientHandler]$httpClientHandler = $NULL
    if ($null -ne $proxy) {
        $httpClientHandler = New-Object -TypeName System.Net.Http.HttpClientHandler
        $httpClientHandler.Proxy = $proxy
        $httpClient = New-Object -TypeName System.Net.Http.HttpClient @($httpClientHandler)
    }
    else {
        $httpClient = New-Object -TypeName System.Net.Http.HttpClient
    }
    return $httpClient
}

function GetLinkPaymentUrl([System.Net.Http.HttpClient]$client, [string]$parameters) {
    $content = New-Object -TypeName System.Net.Http.StringContent($parameters, [System.Text.Encoding]::UTF8, "application/x-www-form-urlencoded")
    $task = $null
    try {
        $url = $apiHost + $apiPath
        $Error.Clear()
        $task = $client.PostAsync($url, $content)
        $task.Wait()
    }
    catch [Exception] {
        return "Error:GetLinkPaymentUrl throw exception"
    }

    $response = $task.Result
    if ($response.StatusCode -eq 200) {
        return $response.Content.ReadAsStringAsync().Result
    }
    else {
        return "Error:" + $response.StatusCode.ToString()
    }
}

function ReadCsv([String]$path) {
    $orderCsv = Import-Csv -Path $path

    $resultList = @()

    $orderCsv | ForEach-Object {
        $orderId = $_.ORDER_ID
        $amount = $_.AMOUNT
        $linkExpDatetime = $_.LINK_EXP_DATETIME
        $settlementType = $_.SETTLEMENT_TYPE
        $settlementSubType = $_.SETTLEMENT_SUB_TYPE
        $timelimitOfPayment = $_.TIMELIMIT_OF_PAYMENT
        $mail = $_.MAIL_ADDRESS
        $name = $_.NAME #メール用
        $paymentMail = $_.PAYMENT_MAIL_CONSUMER_FLAG
        $cardCaptureFlag = $_.CARD_CAPTURE_FLAG
        $mobileEdyShopName = $_.SHOP_NAME
        $suicaScreenTitle = $_.SCREEN_TITLE
        $atmContents = $_.CONTENTS
        $atmContentsKana = $_.CONTENTS_KANA

        $expireDate = $null #メール用
        $expireTime = $null #メール用
        if ([String]::IsNullOrEmpty($linkExpDatetime) -eq $false) {
            $expireDate = ([string]$linkExpDatetime.Substring(0, 4)) + "/" + ([string]$linkExpDatetime.Substring(4, 2)) + "/" + ([string]$linkExpDatetime.Substring(6, 2))
            $expireTime = ([string]$linkExpDatetime.Substring(8, 2)) + ":" + ([string]$linkExpDatetime.Substring(10, 2)) + ":" + ([string]$linkExpDatetime.Substring(12, 2))
        }

        $hash = CreateMerchantHash $merchantHashSeed $merchantId $settlementType $orderId ([string]$amount)
        $sessionId = [string]([Guid]::NewGuid())
        
        $dic = GetParameterDictionary $merchantId $orderId $hash $sessionId ([string]$amount) $linkExpDatetime $settlementType $settlementSubType $cardCaptureFlag $mobileEdyShopName $suicaScreenTitle $atmContents $atmContentsKana $timelimitOfPayment $mail $paymentMail

        $body = GetPostBody($dic)
        $response = GetLinkPaymentUrl $httpClient $body

        $response = $response.trim()

        $result = New-Object PSObject | Select-Object ORDER_ID, LINKPAY_URL_RESULT, URL, MESSAGE, MAIL_RESULT
        $result.ORDER_ID = $orderId

        if ($response.StartsWith("VTW_START_URL=") -eq $true) {
            $result.LINKPAY_URL_RESULT = "success"
            $result.URL = $response.Substring(14)
        }
        elseif ($response.StartsWith("ERROR_MESSAGE=") -eq $true) {
            $result.LINKPAY_URL_RESULT = "failure"
            $result.MESSAGE = $response.Substring(14)
        }
        else {
            $result.LINKPAY_URL_RESULT = "failure"
            $result.MESSAGE = $response
        }

        $orderDate = (Get-Date).ToString("yyyy/MM/dd")

        $mb = CreateMailBody $mailBody $orderId $result.URL $name $amount $orderDate $expireDate $expireTime

        if ($result.LINKPAY_URL_RESULT -eq "success") {
            if ([String]::IsNullOrEmpty($mail) -eq $false) {
                $mailResult = SendMail $mailSubject $mb $mail
                if ($mailResult -eq $true) {
                    $result.MAIL_RESULT = "success"
                }
                else {
                    $result.MAIL_RESULT = "failure"
                }
            }
        }

        $resultList += $result
    }

    $dir = [System.IO.Path]::GetDirectoryName($path)
    $outputFileName = [System.IO.Path]::GetFileNameWithoutExtension($path) + "_" + (Get-Date).ToString("yyMMdd_HHmmss") + "_result.csv"
    $outputPath = Join-Path $dir $outputFileName

    $resultList | Export-Csv $outputPath -NoTypeInformation 

}


[System.Net.Http.HttpClient]$httpClient = $null

if ($useProxy -eq $true) {
    [System.Net.WebProxy]$proxy = $null
    $targetHost = New-Object System.Uri $apiHost

    if ($useProxyCredential -eq $true) {
        $credentials = Get-Credential
        $proxy = GetProxy $targetHost $credentials
    }
    else {
        $proxy = GetProxy $targetHost $null
    }
    $httpClient = GetClient $proxy
}
else {
    $httpClient = GetClient $null
}

if([String]::IsNullOrEmpty($merchantId) -eq $true){
    Write-Host "マーチャントIDが指定されていません。"
    exit
}
if([String]::IsNullOrEmpty($merchantHashSeed) -eq $true){
    Write-Host "ハッシュシードが指定されていません。"
    exit
}

if($Args.Count -eq 1){
    ReadCsv $Args[0]
}
else{
    Write-Host "`r`n・利用方法`r`n"
    Write-Host "第1引数でファイル名を指定してください。"
    Write-Host "> .\csv2linkpay.ps1 .\Book1.csv`r`n"
}
